class Management::UsersController < ManagementController
  before_action :set_user, only: [:show, :edit, :update, :destroy, :change_roles]
  before_action :set_roles, only: [:new, :edit]

  def index
    @users = user_repository.all
  end

  # GET /users/1
  # GET /users/1.json
  def show
  end

  # GET /users/new
  def new
    @user = User.new
  end

  # GET /users/1/edit
  def edit
  end

  # POST /users
  # POST /users.json
  def create
    @user = User.new(user_params)
    @user.roles.ids = role_repository.all.ids

    respond_to do |format|
      if user_repository.save @user
        format.html { redirect_to management_user_path(@user), notice: "User created!" }
        format.json { render :show, status: :created, location: @user }
      else
        format.html { render :new }
        format.json { render json: @user.errors, status: :unprocessable_entity }
      end
    end
  end

  # PATCH/PUT /users/1
  # PATCH/PUT /users/1.json
  def update
    respond_to do |format|
      if user_repository.update(@user, user_params)
        format.html { redirect_to management_user_path(@user), notice: "User updated!" }
      else
        format.html { render :edit }
      end
    end
  end

  # DELETE /users/1
  # DELETE /users/1.json
  def destroy
    user_repository.destroy @user
    respond_to do |format|
      format.html { redirect_to management_users_path, notice: "User destroyed!" }
    end
  end

  def change_roles
    @user.roles = []
    @user.roles << role_repository.find_by_id(params[:role_id])

    respond_to do |format|
      if user_repository.save @user
        format.html {redirect_to :back, notice: "Usuário #{@user.name} definido como #{@user.roles[0].name}"}
      else
        format.html {redirect_to :back, notice: "Erro na atribuição!"}
      end
    end
  end


  def curators
    @curators = Array.new
    @users = User.includes(:roles).all
    @users.all.each do |user|
     @curators << user if user.roles[0].name == "curator"
    end
  end

  def admins
    @admins = Array.new
    @users = User.includes(:roles).all
    @users.all.each do |user|
      @admins << user if user.roles[0].name == "admin"
    end
  end

  private

  # Use callbacks to share common setup or constraints between actions.
  def set_user
    @user = user_repository.find(params[:id])
  end

  # Never trust parameters from the scary internet, only allow the white list through.
  def user_params
    params.require(:user).permit(:name, :email, :password, :password_confirmation, role_ids: [])
  end

  def set_roles
    @roles = role_repository.all
  end

end
