class OrientDb::Base
  include OrientDb::Methods::FinderMethods
  include OrientDb::Methods::SanitizableMethods

  def initialize(orientdb_connection)
    @connection = orientdb_connection
  end

  def connection
    @connection
  end

  def find(id)
    result = get_by_rid(CGI::unescape(id))
    build_object result
  end

  def all
    objects_hash = connection.query "SELECT FROM #{odb_class}", :limit => -1
    objects = build_objects(objects_hash) || []
  end

  def all_from_offset_to_limit(offset, limit)
    objects_hash = connection.query "SELECT FROM #{odb_class} LIMIT #{limit} SKIP #{offset}", :limit => limit
    objects = build_objects(objects_hash) || []
  end

  def destroy(object)
    connection.command sprintf("DELETE VERTEX %s", object.id)
  end

  def create(object)
    if object.valid?
      hash = build_hash(object)
      result = connection.create_document(hash)
      object.id = result["@rid"]
    end
    object
  end

  def update_property(object, property, value)
    if accepted_properties.include? property
      connection.command "UPDATE #{odb_class} SET #{property}='#{value}', last_modified='#{Time.now()}' WHERE @rid = #{object.id}"
    end
  end

  # Take the object and make a hash in the OrientDB format.
  # Used to create a document.
  def build_hash(object)
    hash = {'@class' => odb_class}
    object.instance_variables.each do |var|
      var_name = sanitize_orientdb_vars(var)
      hash[var_name] = sanitize_orientdb_values(object.instance_variable_get(var))
    end
    hash.delete('id')
    # TODO: find a better way to ignore variables from ActiveModel
    hash.delete("validation_context")
    hash.delete("errors")
    hash
  end

  def build_objects(hash=[])
    objects = []
    hash.each do |h|
      objects << build_object(h)
    end
    objects
  end

  def build_object(args={})
    raise NoMethodError, "You must implement this method"
  end

  protected

  def accepted_properties
    raise NoMethodError, "You must implement this method"
  end

  def odb_class
    fullname = self.class.to_s.split('Repository').first
    fullname.split('::').last.to_s
  end

end
