module OrientDb
  class UserRepository < Base
    include OrientDb::Methods::FinderMethods
    include RepositoriesProxy

    def create_graph_node(user)
      begin
        if user.persisted?
          connection.command sprintf("create vertex User set p_id = %d", user.id)
          return true
        end
      rescue Orientdb4r::ServerError => e
        if e.message =~ /ORecordDuplicatedException/
          raise DuplicatedUserIndexError, 'The OrientDB index p_id of User class cannot be duplicated.'
        end
      end

      raise NotPersistedRecordError, 'The user wasn`t persisted yet.'
    end

    def destroy_graph_node(user)
      connection.command sprintf("delete vertex User WHERE p_id = %d", user.id)
    end

    def get_graph_id(user)
      result = connection.query sprintf("SELECT @rid FROM User WHERE p_id = %d", user.id)
      result.first["rid"].to_s
    end

    def is_vertex?(user)
      result = connection.query sprintf("SELECT @rid FROM User WHERE p_id = %d", user.id)
      (result.count > 0)
    end

  end
end
