Rails.application.routes.draw do
  require 'sidekiq/web'

  devise_for :users

  namespace :auth do
    mount_devise_token_auth_for 'User', skip: [:omniauth_callbacks]
  end

  namespace :management do
    root 'welcome#index'
    resource :complaints, except: ['create']
    resources :institutions do
      member do
        post '/user/:user_id', as: :add_user, action: :add_user
        delete '/user/:user_id', as: :remove_user, action: :remove_user
      end
    end
    resources :highlights
    resources :carousels

    resources :statistics do
      collection do
        get :users
        get :collections
        get :accesses
        get :learning_objects
      end
    end

    resources :users do
      resource :admin, only: [:create, :destroy]
      resource :curator, only: [:create, :destroy]

      member do
        post :change_roles
      end

      collection do
        get :curators
        get :admins
      end
    end

    mount Sidekiq::Web, at: '/sidekiq'
  end

  root 'welcome#index'

  resources :subjects, only: [:index, :show]

  resource :complaints, only: ['create']

  resources :institutions do
    member do
      get :users
    end
  end

  resources :learning_objects do
    member do
      post :like
      post :bookmarks
      get :collections
    end
  end

  resources :collections do
    member do
      # collection list
      get :list

      # add a learning object for some collection
      post '/learning_object/:learning_object_id', as: :add_learning_object, action: :add_learning_object

      # remove a learning object for some collection
      delete '/learning_object/:learning_object_id', as: :destroy_learning_object, action: :remove_learning_object

      # change privacy
      post :change_privacy

      # copy / move learning_objects
      get '/copy', as: :copy_learning_objects, action: :copy_learning_objects
      get '/move', as: :move_learning_objects, action: :move_learning_objects
    end
  end

  get '/users/list' => 'users#list', as: :list_all_users
  resources :users, only: [:show] do
    collection do
      get :me
    end
  end

  get '/search' => 'search#index', as: 'search'
  get '/search/fetch' => 'search#fetch', as: 'search_fetch'
  get '/autocomplete' => 'search#autocomplete'

  get '/subject/:id' => 'welcome#subject', as: 'subject_index'
  get '/add_file' => 'welcome#add_file'
  get '/signup' => 'welcome#signup'
  get '/follow_collection' => 'welcome#follow_collection'
  get '/file_outside_user' => 'welcome#file_outside_user'
end
