module OrientDb
  module Methods
    module CountableMethods
      include OrientDb::Methods::EdgeMethods

      #it could perhaps be in the GenericMethods, if we'd like to extend to User counting.
      def get_number_of_non_visualised
        (connection.query("SELECT COUNT(*) FROM LearningObject WHERE in('Views').size() = 0"))[0]["COUNT"].to_i
      end

      def count_all
        Rails.cache.fetch('learning_object/count_all', expires_in: 6.hours) do
          result = connection.query 'SELECT COUNT(*) as count FROM LearningObject', limit: 1
          result[0]["count"]
        end
      end

      def count_likes(learning_object)
        get_in_edges_count "Likes", learning_object.id
      end

      def count_views(learning_object)
        get_in_edges_count "Views", learning_object.id
      end

      def count_downloads(learning_object)
        get_in_edges_count "Downloads", learning_object.id
      end

      def max_likes
        Rails.cache.fetch('max_likes', expires_in: 6.hours) do
          get_max_from_edge("Likes", "in")
        end
      end

      def max_views
        Rails.cache.fetch('max_views', expires_in: 6.hours) do
          get_max_from_edge("Views", "in")
        end
      end

      def max_downloads
        Rails.cache.fetch('max_downloads', expires_in: 6.hours) do
          get_max_from_edge("Downloads", "in")
        end
      end

      def get_max_from_edge(edge_class, type)
        max = connection.command "SELECT max(#{type}('#{edge_class}').size()) FROM LearningObject"
        max["result"][0]["max"]
      end

    end
  end
end
