namespace :bitstream do

  desc "Download Bitstreams"
  task :download_streams => :environment do

    include RepositoriesProxy
    include Bitstream::AcceptedFormats
    include Bitstream::Utils

    # Quantity of items fetched on each iteration
    limit = 1000
    # Start point from where items will be fetched
    offset = 0

    loop do

      begin
        # Get items from dspace (from offset to offset+limit)
        items = learning_object_repository.all_from_offset_to_limit(offset,limit)
      rescue
        # Sleeps for a while to wait database's recovery
        sleep(30.seconds)
        # Goes to next iteration to retry
        next
      else
        # Terminate loop if there are no more items to import
        break if items.empty?

        output_dir = "#{public_dir}/bitstreams/"
        create_dir output_dir

        items.each do |item|
          filename = item.get_filename
          file_format = get_file_extname filename
          if accepted_video_formats.include? file_format
            file_basename = get_file_basename filename
            retrieve_link = item.get_retrievelink
            output_file = output_dir + file_basename
            BitstreamDownloaderWorker.perform_async(retrieve_link,output_file)
          end
        end

        # Increment offset, to get new items on next iteration
        offset += limit
      end
    end
  end

end
